# coffeelint: disable=max_line_length

child_process = require 'child_process'
async = require 'async'
_ = require 'lodash'
chalk = require 'chalk'
debug = require("../../../shared_modules/logger").debug

logger =
  warn: (s) ->
    debug chalk.yellow(s)
  error: (e) ->
    debug chalk.red.bold(e)


Producer = require "../../ProducerClass"

getSystemInfo = (cb) ->
  child_process.exec "LC_ALL=C omgsysteminfo --json 2>/dev/null", (error, stdout, stderr) ->
    logger.error "omgsysteminfo: #{error}" if error
    console.log(stderr) if stderr
    stdout = JSON.parse(stdout) if stdout
    cb null, stdout

getSANs = (cb) ->
  child_process.exec "LC_ALL=C openssl x509 -in /data/cert.pem -noout -text | grep DNS 2>/dev/null", (error, stdout, stderr) ->
    # error code is 1 if grep didn't find anything
    if error and error.code is 1
      cb null, []
      return

    debug error if error
    debug stderr if stderr

    if stdout
      sans = []
      sansandips = stdout.split ","
      _.each sansandips, (sanString) ->
        sanString = sanString.split "DNS:"
        sanPart = sanString[1]
        sans.push sanPart if sanPart
      cb null, sans


getLocalCert = (cb) ->

  child_process.exec 'LC_ALL=C openssl x509 -in /data/cert.pem -noout -subject 2>/dev/null', (error, stdout, stderr) ->
    logger.error "getLocalCert: #{error}" if error
    # console.log stdout if stdout
    console.log stderr if stderr

    if stdout
      certInfo = {}
      stdout = stdout.split("subject= /")[1]
      stdout = stdout.split "/"
      _.each stdout, (certPart) ->
        certPart = certPart.split "="
        certInfo[certPart[0]] = certPart[1]

      getSANs (err, sans) ->
        certInfo.sans = "SANs: #{sans.join ', '}" if sans and sans.length
        cb null, certInfo

    else
      cb null, ""

getCertValidity = (cb) ->
  child_process.exec 'LC_ALL=C [ -e /data/cert.pem ] && openssl x509 -in /data/cert.pem -noout -enddate | awk -F "=" \'{print $2}\' | awk -F " " \'{print $1" "$2" "$4" "$5}\' || echo No Cert found!', (error, stdout, stderr) ->
    if error
      logger.error "cert validity: #{error}"
      cb error
    if stderr
      cb stderr
    if stdout
      stdout = stdout.replace "\n",""
      cb null, stdout

getMultiple = (commandObject, cb) ->
  args = commandObject.command.split " "
  command = args.shift()
  child = child_process.spawn "#{command}", args#, {env: {"LC_ALL": "C"}}

  returnObject = {}

  child.stdout.on "data", (stdout) ->
    stdout = stdout.toString()
    # stdout = stdout.replace /ilidloop\:\ 20ms.*/g, "fo"
    returnObject[commandObject.name] = returnObject[commandObject.name] or ""
    returnObject[commandObject.name] += stdout
  child.stderr.on "data", (stderr) ->
    logger.warn stderr
  child.on "error", (error) ->
    logger.error error + commandObject.command

  child.on "close", () ->
    cb null, returnObject

multipleCommandsToRun = [
  {name: "ifconfig", command:"ifconfig -a"},
  {name: "pscallp", command: "pscallp"},
  {name: "dmesg", command: "dmesg"},
  {name: "procs", command: "ps ax"},
  {name: "netstat", command: "netstat -an --inet"}
]

class SystemInfo extends Producer
  isRunning = false
  constructor: ->
    super "systeminfo",15 * 1000

    run = =>
      someCommands = (cb) =>
        async.map multipleCommandsToRun, getMultiple, (err, results) =>
          if err
            logger.error err
            @sendError err
            return

          returnObject = {}
          _.each results, (object) ->
            _.each object, (data, propName) ->
              returnObject[propName] = data
          cb null, returnObject

      async.parallel {
        systeminfo: getSystemInfo
        sslcert: getLocalCert
        sslvalid: getCertValidity
        multi: someCommands
      }, (err, results) =>
        {systeminfo, sslcert, sslvalid, multi} = results
        systeminfo.ifconfig = multi.ifconfig
        systeminfo.pscallp = multi.pscallp
        systeminfo.dmesg = multi.dmesg
        systeminfo.procs = multi.procs
        systeminfo.netstat = multi.netstat

        systeminfo.sslcert = {Company: sslcert, "valid until": sslvalid}
        systeminfo = JSON.stringify(systeminfo)

        while systeminfo.match /Unknown\ Unknown\ Unknown\ Unknown/g
          systeminfo = systeminfo.replace /Unknown\ Unknown/gi, "Unknown"

        while systeminfo.match /Unknown\ Processor\ \ /g
          systeminfo = systeminfo.replace /Unknown\ Processor\ \ /g, "Unknown Processor "

        while systeminfo.match /000000000000\ 000000000000\ /g
          systeminfo = systeminfo.replace /000000000000\ 000000000000\ /g, "000000000000\ "

        while systeminfo.match /Unknown\ Processor\ Unknown\ Processor\ Unknown\ Processor\ Unknown\ Processor/g
          systeminfo = systeminfo.replace /Unknown\ Processor\ Unknown\ Processor/gi, "Unknown Processor"

        systeminfo = JSON.parse(systeminfo)

        @sendData systeminfo

    #run once on load
    run()

    getNewData = false
    @start = =>
      @debug "starting"
      #run directly when started
      run()
      clearInterval getNewData if getNewData
      getNewData = setInterval run, 20 * 1000

    @stop = ->
      clearInterval getNewData if getNewData
      isRunning = false

  subscribe: =>
    unless isRunning
      isRunning = true
      @start()
    super

module.exports = new SystemInfo()
